@@ -22,12 +22,10 @@ gmt begin ex33
 	# and stack these using the median, write stacked profile
 	gmt grdtrack ridge.txt -Gspac_33.nc -C400k/2k/10k+v -Sm+sstack.txt > table.txt
 	gmt plot -W0.5p table.txt
-	# Show upper/lower values encountered as an envelope
-	gmt convert stack.txt -o0,5 > env.txt
-	gmt convert stack.txt -o0,6 -I -T >> env.txt
-	gmt plot -R-200/200/-3500/-2000 -JX15c/7.5c -Glightgray env.txt -Yh+3c
-	gmt plot -W3p stack.txt -Bxafg1000+l"Distance from ridge (km)" -Byaf+l"Depth (m)" -BWSne
+	# Show upper/lower 2-sigma confidence bounds encountered as an envelope
+	gmt plot -R-200/200/-3500/-2000 -JX15c/7.5c -W3p stack.txt -i0,1,5,6 -L+b -Glightgray -Yh+3c
+	gmt basemap  -Bxafg1000+l"Distance from ridge (km)" -Byaf+l"Depth (m)" -BWSne
 	echo "0 -2000 MEDIAN STACKED PROFILE" | gmt text -Gwhite -F+jTC+f14p -Dj8p
 	# cleanup
-	rm -f ridge.txt table.txt env.txt stack.txt spac_33.nc
+	rm -f ridge.txt table.txt stack.txt spac_33.nc
 gmt end show
