@@ -16,13 +16,12 @@ gmt begin ex39
 	gmt sph2grd @VenusTopo180.txt -I1 -Rg -Ng -Gv2.nc -F1/1/85/90
 	gmt sph2grd @VenusTopo180.txt -I1 -Rg -Ng -Gv3.nc -F1/1/170/180
 	gmt grd2cpt v3.nc -Crainbow -E
-	gmt grdimage v1.nc -I+a45+nt0.75 -JG90/30/5i -Bg -X3i -Y1.1i
-	echo 4 4.5 L = 30 | gmt text -R0/6/0/6 -Jx1i -Dj0.2i -F+f16p+jLM -N
-	gmt colorbar --FORMAT_FLOAT_MAP="%'g" -Dx1.25i/-0.2i+jTC+w5.5i/0.1i+h -Bxaf -By+lm
-	gmt grdimage v2.nc -I+a45+nt0.75 -JG -Bg -X-1.25i -Y1.9i
-	echo 4 4.5 L = 90 | gmt text -R0/6/0/6 -Jx1i -Dj0.2i -F+f16p+jLM -N
-	gmt grdimage v3.nc -I+a45+nt0.75 -JG -Bg -X-1.25i -Y1.9i
-	echo 4 4.5 L = 180 | gmt text -R0/6/0/6 -Jx1i -Dj0.2i -F+f16p+jLM -N
-	echo 3.75 5.4 Venus Spherical Harmonic Model | gmt text -F+f24p+jCM -N
+	gmt grdimage v1.nc -I+a45+nt0.75 -JG90/30/12c -Bg -X7.5c
+	echo L = 30 | gmt text -F+f16p+cTR
+	gmt colorbar --FORMAT_FLOAT_MAP="%'g" -Dx3c/-0.5c+jTC+w13c/0.25c+h  -Bxaf -By+lm
+	gmt grdimage v2.nc -I+a45+nt0.75 -JG -Rg -Bg -X-3c -Y5c
+	echo L = 90 | gmt text -F+f16p+cTR -B+t"Venus Spherical Harmonic Model" --MAP_TITLE_OFFSET=5.5c
+	gmt grdimage v3.nc -I+a45+nt0.75 -JG -Rg -Bg -X-3c -Y5c
+	echo L = 180 | gmt text -F+f16p+cTR
 	rm -f v?.nc
 gmt end show
