@@ -13,7 +13,7 @@ plat=40S
 w2=$(gmt math -Q $w 2 DIV =)
 h2=$(gmt math -Q $h 2 DIV =)
 R=-R-${w2}/$w2/-${h2}/${h2}+uk
-gmt coast $R -JOA$lon/$lat/$az/3i -Ba5f5g5 -Gred -Dh -TdjBR+w0.5i+l+o0.2i/-0.05i --FONT_TITLE=9p --MAP_ANNOT_OBLIQUE=separate,lon_horizontal,lat_parallel --FORMAT_GEO_MAP=dddF
+gmt coast $R -JOA$lon/$lat/$az/3i -Ba5f5g5 -Gred -Dh -TdjBR+w0.5i+l --MAP_ANNOT_OBLIQUE=separate,lon_horizontal,lat_parallel --FORMAT_GEO_MAP=dddF
 echo $plon $plat | gmt plot -Sc0.2c -Gblue -W0.25p
 gmt plot -R0/3/0/1.5 -Jx1i -W0.25p,- << EOF
 >
@@ -32,9 +32,8 @@ gmt text -F+f12p,Times-Italic+j -Dj0.1i -Gwhite << EOF
 3	0.75	TR	x
 1.5	1.5	TR	y
 EOF
-#echo $plon $plat | gmt mapproject -JoA$lon/$lat/$az/1:1 -Fk -C
 az=215
-gmt coast $R -JOA$lon/$lat/$az/3i -Ba5f5g5 -Gred -Dh -X3.4i -TdjTL+w0.5i+l+o0.2i/-0.05i --FONT_TITLE=9p --MAP_ANNOT_OBLIQUE=separate,lon_horizontal,lat_parallel --FORMAT_GEO_MAP=dddF
+gmt coast $R -JOA$lon/$lat/$az/3i -Ba5f5g5 -Gred -Dh -X3.4i -TdjTL+w0.5i+l --MAP_ANNOT_OBLIQUE=separate,lon_horizontal,lat_parallel --FORMAT_GEO_MAP=dddF
 echo $plon $plat | gmt plot -Sc0.2c -Gblue -W0.25p
 gmt plot -R0/3/0/1.5 -Jx1i -W0.25p,- << EOF
 >
@@ -53,5 +52,4 @@ gmt text -F+f12p,Times-Italic+j -Dj0.1i -Gwhite << EOF
 3	0.75	TR	x
 1.5	1.5	TR	y
 EOF
-#echo $plon $plat | gmt mapproject -JoA$lon/$lat/$az/1:1 -Fk -C
 gmt end show
