@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -355,7 +355,7 @@ int gmtlib_is_esri_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 
 	if (!strcmp (HH->name, "="))
 		return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
-	if ((e = gmt_get_ext (HH->name)) && !strcmp (e, GMT_TILE_EXTENSION_REMOTE)) return (-1);	/* Watch out for .jp2 tiles since they may contain W|E|S|N codes as well and the ESRI check comes before GDAL check*/
+	if ((e = gmt_get_ext (HH->name)) && !strcmp (e, GMT_TILE_EXTENSION_REMOTE)) return (-1);	/* Watch out for .jp2 tiles since they may contain W|E|S|N codes as well and the ESRI check comes before GDAL check */
 	if ((fp = gmt_fopen (GMT, HH->name, "r")) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);
 	if (fgets (record, GMT_BUFSIZ, fp) == NULL) {	/* Just get first line. Not using gmt_fgets since we may be reading a binary file */
@@ -532,10 +532,11 @@ int gmt_esri_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, gmt
 	if (pad[XHI] > 0) width_out += pad[XHI];
 	n_expected = header->n_columns;
 
-	if (nBits == 32)		/* Either an ASCII file or ESRI .HDR with NBITS = 32, in which case we assume it's a file of floats */
-		tmp = gmt_M_memory (GMT, NULL, n_expected, float);
-	else
-		tmp16 = gmt_M_memory (GMT, NULL, n_expected, int16_t);
+	if (nBits == 32) {		/* Either an ASCII file or ESRI .HDR with NBITS = 32, in which case we assume it's a file of floats */
+		if ((tmp = gmt_M_memory (GMT, NULL, n_expected, float)) == NULL) return GMT_MEMORY_ERROR;
+	}
+	else if ((tmp16 = gmt_M_memory (GMT, NULL, n_expected, int16_t)) == NULL)
+		return GMT_MEMORY_ERROR;
 
 	header->z_min = DBL_MAX;	header->z_max = -DBL_MAX;
 	HH->has_NaNs = GMT_GRID_NO_NANS;	/* We are about to check for NaNs and if none are found we retain 1, else 2 */
