@@ -1,16 +1,16 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
- *      This program is free software; you can redistribute it and/or modify
- *      it under the terms of the GNU Lesser General Public License as published by
- *      the Free Software Foundation; version 3 or any later version.
+ *	This program is free software; you can redistribute it and/or modify
+ *	it under the terms of the GNU Lesser General Public License as published by
+ *	the Free Software Foundation; version 3 or any later version.
  *
- *      This program is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU Lesser General Public License for more details.
+ *	This program is distributed in the hope that it will be useful,
+ *	but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *	GNU Lesser General Public License for more details.
  *
  *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
@@ -448,7 +448,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 	/* Get some metadata for each band. */
 	/* ------------------------------------------------------------------------- */
 
-	Ctrl->band_field_names = gmt_M_memory(GMT, NULL, raster_count, struct GDAL_BAND_FNAMES);
+	if ((Ctrl->band_field_names = gmt_M_memory(GMT, NULL, raster_count, struct GDAL_BAND_FNAMES)) == NULL) return GMT_NOTSET;
 
 	/* ==================================================================== */
 	/*      Loop over bands.                                                */
@@ -536,7 +536,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 		&& (hTable = GDALGetRasterColorTable(hBand)) != NULL) {
 
 		Ctrl->nIndexedColors = GDALGetColorEntryCount(hTable);
-		Ctrl->ColorMap = gmt_M_memory(GMT, NULL, Ctrl->nIndexedColors*4+1, int);
+		if ((Ctrl->ColorMap = gmt_M_memory(GMT, NULL, Ctrl->nIndexedColors*4+1, int)) == NULL) return GMT_NOTSET;
 		for (i = 0; i < Ctrl->nIndexedColors; i++) {
 			GDALGetColorEntryAsRGB(hTable, i, &sEntry);
 			gmt_M_set_rgba (Ctrl->ColorMap, i, 0, Ctrl->nIndexedColors, sEntry.c1);
@@ -554,7 +554,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 		if (pszNBits && !strcmp(pszNBits, "1")) {
 			/* Create a two-color index table for black and white only */
 			Ctrl->nIndexedColors = 2;
-			Ctrl->ColorMap = gmt_M_memory(GMT, NULL, Ctrl->nIndexedColors*4+1, int);
+			if ((Ctrl->ColorMap = gmt_M_memory(GMT, NULL, Ctrl->nIndexedColors*4+1, int)) == NULL) return GMT_NOTSET;
 			for (j = 0; j < 3; j++) {	/* For R, G, B */
 				gmt_M_set_rgba (Ctrl->ColorMap, 0, j, Ctrl->nIndexedColors, 0);
 				gmt_M_set_rgba (Ctrl->ColorMap, 1, j, Ctrl->nIndexedColors, 255);
@@ -684,16 +684,16 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 		Ctrl->hdr[7] = adfGeoTransform[1];
 		Ctrl->hdr[8] = fabs(adfGeoTransform[5]);
 
+		if (got_R) {
+			Ctrl->hdr[0] = dfULX;	Ctrl->hdr[1] = dfLRX;
+			Ctrl->hdr[2] = dfLRY;	Ctrl->hdr[3] = dfULY;
+		}
+
 		if (Ctrl->hdr[2] > Ctrl->hdr[3]) {	/* Sometimes GDAL does it: y_min > y_max. If so, revert it */
 			tmpdble = Ctrl->hdr[2];
 			Ctrl->hdr[2] = Ctrl->hdr[3];
 			Ctrl->hdr[3] = tmpdble;
 		}
-
-		if (got_R) {
-			Ctrl->hdr[0] = dfULX;	Ctrl->hdr[1] = dfLRX;
-			Ctrl->hdr[2] = dfLRY;	Ctrl->hdr[3] = dfULY;
-		}
 	}
 
 	GDALClose (hDataset);
@@ -756,7 +756,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 	bool   pixel_reg = false;	/* GDAL decides everything is pixel reg, we make our decisions based on data type */
 	bool   fliplr, got_R = false, got_r = false;
 	bool   topdown = false, rowmajor = true;               /* arrays from GDAL have this order */
-	bool   just_copy = false, copy_flipud = false;
+	bool   just_copy = false, copy_flipud = false, UDflip_Y = false;
 	int	   *whichBands = NULL;
 	int64_t *rowVec = NULL, *colVec = NULL;
 	int64_t  off, i_x_nXYSize, startColPos = 0, indent = 0, col_indent = 0, nXSize_withPad = 0, nYSize_withPad;
@@ -775,7 +775,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 	gmt_M_memset (anSrcWin, 4, int);
 
 	if (prhs->B.active) {		/* We have a selected bands request */
-		whichBands = gmt_M_memory (GMT, NULL, 128, int);		/* 128 is huge enough */
+		if ((whichBands = gmt_M_memory (GMT, NULL, 128, int)) == NULL) return GMT_NOTSET;		/* 128 is huge enough */
 		nReqBands = gdal_decode_columns (GMT, prhs->B.bands, whichBands);
 	}
 	else if (prhs->f_ptr.active) {
@@ -944,6 +944,12 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 		GMT_Report (GMT->parent, GMT_MSG_INFORMATION, "gmt_gdalread: Image %s does not have geographic coordinates so -R is ignored for subregion\n", gdal_filename);
 		got_R = false;
 	}
+	if ((dfLRY < dfULY) && adfGeoTransform[5] > 0) {
+		/* Case when y-axes comes positive up from GDAL (that normally has it positive down) */
+		double t;
+		t = dfLRY;	dfLRY = dfULY;	dfULY = t;
+		UDflip_Y = true;
+	}
 
 	if (got_R || got_r) {
 		/* -------------------------------------------------------------------- */
@@ -1051,8 +1057,8 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 		case GDT_Byte:
 			if (prhs->c_ptr.active)	/* We have a pointer with already allocated memory ready to use */
 				Ctrl->UInt8.data = prhs->c_ptr.grd;
-			else
-				Ctrl->UInt8.data = gmt_M_memory (GMT, NULL, n_alloc, uint8_t); /* aka unsigned char */
+			else if ((Ctrl->UInt8.data = gmt_M_memory (GMT, NULL, n_alloc, uint8_t)) == NULL)
+				return GMT_NOTSET; /* aka unsigned char */
 
 			if (do_BIP) {
 				if (nBands == 4)	/* Assume fourth band holds the alpha channel */
@@ -1068,8 +1074,8 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				Ctrl->Float.data = prhs->f_ptr.grd;
 			else if (prhs->c_ptr.active) 	/* Use the previously allocated pointer */
 				Ctrl->Int16.data = (int16_t *)prhs->c_ptr.grd;
-			else
-				Ctrl->Int16.data = gmt_M_memory (GMT, NULL, n_alloc, int16_t);
+			else if ((Ctrl->Int16.data = gmt_M_memory (GMT, NULL, n_alloc, int16_t)) == NULL)
+				return GMT_NOTSET;
 			break;
 		case GDT_UInt16:
 			if (prhs->f_ptr.active) {	/* Use the previously allocated float pointer */
@@ -1080,24 +1086,24 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				Ctrl->UInt16.data = (uint16_t *)prhs->c_ptr.grd;
 				Ctrl->UInt16.active = true;		/* In case it was not set yet */
 			}
-			else
-				Ctrl->UInt16.data = gmt_M_memory (GMT, NULL, n_alloc, uint16_t);
+			else if ((Ctrl->UInt16.data = gmt_M_memory (GMT, NULL, n_alloc, uint16_t)) == NULL)
+				return GMT_NOTSET;
 			break;
 		case GDT_Int32:
 			if (prhs->f_ptr.active)		/* Use the previously allocated float pointer */
 				Ctrl->Float.data = prhs->f_ptr.grd;
 			else if (prhs->c_ptr.active) 	/* Use the previously allocated pointer */
 				Ctrl->Int32.data = (int32_t *)prhs->c_ptr.grd;
-			else
-				Ctrl->Int32.data = gmt_M_memory (GMT, NULL, n_alloc, int32_t);
+			else if ((Ctrl->Int32.data = gmt_M_memory (GMT, NULL, n_alloc, int32_t)) == NULL)
+				return GMT_NOTSET;
 			break;
 		case GDT_UInt32:
 			if (prhs->f_ptr.active)		/* Use the previously allocated float pointer */
 				Ctrl->Float.data = prhs->f_ptr.grd;
 			else if (prhs->c_ptr.active) 	/* Use the previously allocated pointer */
 				Ctrl->UInt32.data = (uint32_t *)prhs->c_ptr.grd;
-			else
-				Ctrl->UInt32.data = gmt_M_memory (GMT, NULL, n_alloc, uint32_t);
+			else if ((Ctrl->UInt32.data = gmt_M_memory (GMT, NULL, n_alloc, uint32_t)) == NULL)
+				return GMT_NOTSET;
 			break;
 		case GDT_Float32:
 		case GDT_Float64:
@@ -1108,6 +1114,8 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 					Ctrl->Float.data = gmt_M_memory (GMT, NULL, n_alloc, float);
 				else
 					Ctrl->Float.data = gmt_M_memory (GMT, NULL, 2 * n_alloc, float);
+				if (Ctrl->Float.data == NULL)
+					return GMT_NOTSET;
 			}
 			break;
 		default:
@@ -1146,9 +1154,8 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 		}
 
 		/* Can't find why but have to multiply by nRGBA otherwise it crashes in win32 builds */
-		rowVec = gmt_M_memory(GMT, NULL, (nRowsPerBlock * nRGBA) * nBlocks, size_t);
-		for (m = 0; m < nY; m++) rowVec[m] = m * nX[piece];	/* Steps in pixels as we go down rows */
-		colVec = gmt_M_memory(GMT, NULL, (nX[piece]+pad_w[piece]+pad_e[piece]) * nRGBA, size_t);	/* For now this will be used only to select BIP ordering */
+		if ((rowVec = gmt_M_memory(GMT, NULL, (nRowsPerBlock * nRGBA) * nBlocks, size_t)) == NULL) return GMT_NOTSET;
+		if ((colVec = gmt_M_memory(GMT, NULL, (nX[piece]+pad_w[piece]+pad_e[piece]) * nRGBA, size_t)) == NULL) return GMT_NOTSET;	/* For now this will be used only to select BIP ordering */
 		for (i = 0; i < nBands; i++) {
 			if (!nReqBands)		/* No band selection, read them sequentially */
 				hBand = GDALGetRasterBand(hDataset, i+1);
@@ -1182,7 +1189,12 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				row_i = k * nRowsPerBlock;
 				row_e = (k + 1) * nRowsPerBlock;
 				buffy = nRowsPerBlock;
-				for (m = 0; m < nRowsPerBlock; m++) rowVec[m+k*nRowsPerBlock] = m * nX[piece];
+				if (UDflip_Y) {		/* Case when y-axes comes positive up from GDAL (that normally has it positive down) */
+					size_t n = ((nRowsPerBlock-1) + (nBlocks-1) * nRowsPerBlock);
+					for (m = nRowsPerBlock-1; m >= 0 ; m--) rowVec[n - (m + k * nRowsPerBlock)] = m * nX[piece];
+				}
+				else
+					for (m = 0; m < nRowsPerBlock; m++) rowVec[m + k * nRowsPerBlock] = m * nX[piece];
 				if (k == nBlocks-1) {					/* Last block only by chance is not smaller than the others */
 					buffy = nBufYSize - k * nRowsPerBlock;
 					row_e = nYSize;
