@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,7 @@
 
 /*!
  * \file gmt_io.h
- * \brief  
+ * \brief
  */
 
 #ifndef GMT_IO_H
@@ -126,10 +126,9 @@ enum GMT_lon_enum {
 
 /*! How to handle NaNs in records */
 enum GMT_io_nan_enum {
-	GMT_IO_NAN_OK = 0,	/* NaNs are fine; just output the record as is */
-	GMT_IO_NAN_SKIP,	/* -s[cols]	: Skip records with z == NaN in selected cols [z-col only] */
-	GMT_IO_NAN_KEEP,	/* -sr		: Skip records with z != NaN */
-	GMT_IO_NAN_ONE};	/* -sa		: Skip records with at least one NaN */
+	GMT_IO_NAN_SKIP = 1,	/* -s[cols]	: Skip records with z == NaN in selected cols [col=2 only] */
+	GMT_IO_NAN_KEEP = 2,	/* -s+r		: Skip records with z != NaN */
+	GMT_IO_NAN_ANY  = 4};	/* -s+a		: Skip records with at least one NaN [skip records with all cols = NaN] */
 
 /* Use POSIX functions ftello() and fseeko(), which represent the
  * position using the off_t type: */
@@ -200,7 +199,7 @@ struct GMT_GEO_IO {			/* For geographic output and plotting */
 	unsigned int range;		/* 0 for 0/360, 1 for -360/0, 2 for -180/+180 */
 	unsigned int wesn;		/* 1 if we want sign encoded with suffix W, E, S, N, 2 if also want space before letter */
 	int order[3];			/* The relative order of degree, minute, seconds in form (-ve if unused) */
-	bool decimal;			/* true if we want to use the D_FORMAT for decimal degrees only */
+	bool decimal;			/* true if we want to use the FORMAT_FLOAT_OUT for decimal degrees only */
 	bool no_sign;			/* true if we want absolute values (plot only) */
 	char x_format[GMT_LEN64];	/* Actual C format used to plot/output longitude */
 	char y_format[GMT_LEN64];	/* Actual C format used to plot/output latitude */
@@ -212,7 +211,7 @@ struct GMT_COL_INFO {	/* Used by -i and input parsing */
 	unsigned int order;	/* The initial order (0,1,...) but this will be sorted on col */
 	unsigned int convert;	/* 2 if we must convert the data by log10, 1 if scale, offset */
 	double scale;		/* Multiplier for raw in value */
-	double offset;		/* Offset applied after multiplier */ 
+	double offset;		/* Offset applied after multiplier */
 };
 
 struct GMT_COL_TYPE {	/* Used by -b for binary formatting */
@@ -221,6 +220,16 @@ struct GMT_COL_TYPE {	/* Used by -b for binary formatting */
 	int (*io) (struct GMT_CTRL *, FILE *, uint64_t, double *);	/* Pointer to the correct read or write function given type/swab */
 };
 
+/*! For selecting row ranges via -q */
+struct GMT_ROW_RANGE {
+	int64_t first, last, inc;
+};
+
+/*! For selecting data ranges via -q */
+struct GMT_DATA_RANGE {
+	double first, last;
+};
+
 struct GMT_IO {				/* Used to process input data records */
 	void * (*input) (struct GMT_CTRL *, FILE *, uint64_t *, int *);	/* Pointer to function reading ASCII or binary tables */
 	int (*output) (struct GMT_CTRL *, FILE *, uint64_t, double *, char *);	/* Pointer to function writing ASCII or binary tables */
@@ -241,11 +250,15 @@ struct GMT_IO {				/* Used to process input data records */
 	bool skip_duplicates;	/* true if we should ignore duplicate x,y records */
 	bool variable_in_columns;	/* true if we are reading ASCII records with variable numbers of columns */
 	bool need_previous;		/* true if when parsing a record we need access to previous record values (e.g., for gap or duplicate checking) */
+	bool has_previous_rec;		/* true if we have the previous record for this segment */
 	bool warn_geo_as_cartesion;	/* true if we should warn if we read a record with geographic data while the expected format has not been set (i.e., no -J or -fg) */
 	bool first_rec;			/* true when reading very first data record in a dataset */
 	bool trailing_text[2];	/* Default is to process training text unless turned off via -i, -o */
-	bool hash_refreshed;		/* true after calling the hash_refresh function the first time */
+	bool refreshed[2];		/* true after calling the hash_refresh function the first time, for hash and info, respectively */
+	bool new_data_list;		/* true after when a server refresh yields an updated gmt_data_server.txt */
 	bool internet_error;		/* true after failing to get hash table due to time-out */
+	bool grid_padding;		/* If true we try to read two extra rows/cols from grids for BC purposes */
+	bool leave_as_jp2;		/* If true we do not convert downloaded JP2 grids to NC right away, but as needed */
 	uint64_t seg_no;		/* Number of current multi-segment in entire data set */
 	uint64_t seg_in_tbl_no;		/* Number of current multi-segment in current table */
 	uint64_t n_clean_rec;		/* Number of clean records read (not including skipped records or comments or blanks) */
@@ -255,6 +268,7 @@ struct GMT_IO {				/* Used to process input data records */
 	unsigned int record_type[2];	/* Either GMT_READ|WRITE_DATA (0), GMT_READ|WRITE_TEXT (1), or GMT_READ|WRITE_MIXED (2) (for input and output) */
 	unsigned int n_numerical_cols;	/* As it says */
 	unsigned int max_cols_to_read;	/* For ascii input [all] */
+	unsigned int n_row_ranges[2];	/* How many row ranges given in -q */
 	enum GMT_ogr_status ogr;	/* Tells us if current input source has OGR/GMT metadata (GMT_OGR_TRUE) or not (GMT_OGR_FALSE) or not set (GMT_OGR_UNKNOWN) */
 	unsigned int status;		/* 0	All is ok
 					   1	Current record is segment header
@@ -263,7 +277,9 @@ struct GMT_IO {				/* Used to process input data records */
 					   8	NaNs encountered in first 2/3 cols */
 	uint64_t rec_no;		/* Number of current records (counts headers etc) in entire data set */
 	uint64_t rec_in_tbl_no;		/* Number of current record (counts headers etc) in current table */
-	uint64_t pt_no;			/* Number of current valid points in a row  */
+	uint64_t data_record_number_in_set[2];	/* Number of current valid data record number in the whole dataset, for input and output. Headers not counted.  */
+	uint64_t data_record_number_in_tbl[2];	/* Number of current valid data record number in the current table, for input and output. Headers not counted.  */
+	uint64_t data_record_number_in_seg[2];	/* Number of current valid data record number in the current segment, for input and output. Headers not counted.  */
 	int64_t curr_pos[2][4];		/* Keep track of current input/output table, segment, row, and table headers (for rec-by-rec action) */
 	char r_mode[4];			/* Current file opening mode for reading (r or rb) */
 	char w_mode[4];			/* Current file opening mode for writing (w or wb) */
@@ -272,9 +288,7 @@ struct GMT_IO {				/* Used to process input data records */
 	char curr_trailing_text[GMT_BUFSIZ];	/* Current text portion of current record (or NULL) */
 	char segment_header[GMT_BUFSIZ];	/* Current ASCII segment header */
 	char filename[2][PATH_MAX];	/* Current filenames (or <stdin>/<stdout>) */
-#ifdef HAVE_GDAL
 	char tempfile[PATH_MAX];	/* Temporary file used to read - should be removed when closed */
-#endif
 	char col_set[2][GMT_MAX_COLUMNS];	/* Keeps track of which columns have had their type set */
 	char *o_format[GMT_MAX_COLUMNS];	/* Custom output ASCII format to overrule format_float_out */
 	int ncid;			/* NetCDF file ID (when opening netCDF file) */
@@ -295,10 +309,21 @@ struct GMT_IO {				/* Used to process input data records */
 	unsigned int io_nan_col[GMT_MAX_COLUMNS];	/* Array of columns to consider for -s option ir true */
 	struct GMT_COL_INFO col[2][GMT_MAX_COLUMNS];	/* Order of columns on input and output unless 0,1,2,3,... */
 	struct GMT_COL_TYPE fmt[2][GMT_MAX_COLUMNS];	/* Formatting information for binary data */
+	struct GMT_ROW_RANGE row_range[2][GMT_MAX_RANGES];		/* One or more ranges for input or output rows */
+	struct GMT_DATA_RANGE data_range[2][GMT_MAX_RANGES];		/* One or more ranges for input or output times */
 	struct GMT_OGR *OGR;		/* Pointer to GMT/OGR info used during reading */
 	struct GMT_RECORD record;	/* Current record with pointers to data columns and text */
+	double *nc_xarray, *nc_yarray;	/* For grids with variable x,y arrays */
+	enum GMT_time_period cycle_operator;
+	bool cycle_interval;	/* true for annual and weekly cycles */
+	int64_t cycle_col;	/* The input column with periodic time [-1 meaning no such thing] */
+	double cycle_min;	/* Min cyclical time requested via -R */
+	double cycle_max;	/* Max cyclical time requested via -R */
+	double cycle_range;	/* A full period of time */
+	double cycle_period;	/* Custom period */
+	double cycle_phase;		/* Custom phase */
 	/* The remainder are just pointers to memory allocated elsewhere */
-	int *varid;			/* Array of variable IDs (netCDF only) */
+	int *grpid, *varid;			/* Arrays of group and variable IDs (netCDF only) */
 	double *scale_factor;		/* Array of scale factors (netCDF only) */
 	double *add_offset;		/* Array of offsets (netCDF only) */
 	double *missing_value;		/* Array of missing values (netCDF only) */
