@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -2055,7 +2055,7 @@ void gmt_getmad (struct GMT_CTRL *GMT, double *x, uint64_t n, double location, d
 		return;
 	}
 
-	dev = gmt_M_memory (GMT, NULL, n, double);
+	if ((dev = gmt_M_memory (GMT, NULL, n, double)) == NULL) return;
 	for (i = 0; i < n; i++) dev[i] = fabs (x[i] - location);
 	gmt_sort_array (GMT, dev, n, GMT_DOUBLE);
 	/* Eliminate any NaNs which would have congregated at the end of the array */
@@ -2081,7 +2081,7 @@ void gmt_getmad_f (struct GMT_CTRL *GMT, gmt_grdfloat *x, uint64_t n, double loc
 		*scale = 0.0;
 		return;
 	}
-	dev = gmt_M_memory (GMT, NULL, n, double);
+	if ((dev = gmt_M_memory (GMT, NULL, n, double)) == NULL) return;
 	for (i = 0; i < n; i++) dev[i] = (gmt_grdfloat) fabs (x[i] - location);
 	gmt_sort_array (GMT, dev, n, GMT_FLOAT);
 	for (i = n; i > 0 && gmt_M_is_fnan (dev[i-1]); i--);
@@ -2498,7 +2498,7 @@ void gmt_PvQv (struct GMT_CTRL *GMT, double x, double v_ri[], double pq[], unsig
 double gmt_grd_mean (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *W) {
 	/* Compute the [weighted] mean of a grid.  Handle geographic grids with spherical weights W [NULL for cartesian] */
 	uint64_t node, n = 0;
-	unsigned int row, col;
+	openmp_int row, col;
 	double sum_zw = 0.0, sum_w = 0.0;
 	if (W) {	/* Weights provided */
 		gmt_M_grd_loop (GMT, G, row, col, node) {
@@ -2522,7 +2522,7 @@ double gmt_grd_mean (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *
 double gmt_grd_std (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *W) {
 	/* Compute the [weighted] std of a grid.  Handle geographic grids with spherical weights W [NULL for cartesian] */
 	uint64_t node, n = 0;
-	unsigned int row, col;
+	openmp_int row, col;
 	double std, mean = 0.0, delta, sumw = 0.0;
 	if (W) {	/* Weights provided */
 		double temp, R, M2 = 0.0;
@@ -2554,7 +2554,7 @@ double gmt_grd_std (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *W
 double gmt_grd_rms (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *W) {
 	/* Compute the [weighted] rms of a grid.  Handle geographic grids with spherical weights W [NULL for cartesian] */
 	uint64_t node, n = 0;
-	unsigned int row, col;
+	openmp_int row, col;
 	double rms, sum_z2w = 0.0, sum_w = 0.0;
 	if (W) {	/* Weights provided */
 		gmt_M_grd_loop (GMT, G, row, col, node) {
@@ -2582,8 +2582,9 @@ double gmt_grd_median (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID
 	double wmed;
 
 	if (W) {	/* Weights provided */
-		unsigned int row, col;
+		openmp_int row, col;
 		struct GMT_OBSERVATION *pair = gmt_M_memory (GMT, NULL, G->header->nm, struct GMT_OBSERVATION);
+		if (pair == NULL) return 0.0;
 		/* 1. Create array of value,weight pairs, skipping NaNs */
 		gmt_M_grd_loop (GMT, G, row, col, node) {
 			if (gmt_M_is_fnan (G->data[node]) || gmt_M_is_dnan (W->data[node]))
@@ -2616,8 +2617,9 @@ double gmt_grd_mad (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *W
 	uint64_t node, n = 0;
 	double wmed, wmad;
 	if (W) {	/* Weights provided */
-		unsigned int row, col;
+		openmp_int row, col;
 		struct GMT_OBSERVATION *pair = gmt_M_memory (GMT, NULL, G->header->nm, struct GMT_OBSERVATION);
+		if (pair == NULL) return 0.0;
 		if (median) {	/* Already have the median */
 			wmed = *median;
 			/* 3. Compute the absolute deviations from this median */
@@ -2673,8 +2675,9 @@ double gmt_grd_mode (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *
 	double wmode;
 
 	if (W) {	/* Weights provided */
-		unsigned int row, col;
+		openmp_int row, col;
 		struct GMT_OBSERVATION *pair = gmt_M_memory (GMT, NULL, G->header->nm, struct GMT_OBSERVATION);
+		if (pair == NULL) return 0.0;
 		/* 1. Create array of value,weight pairs, skipping NaNs */
 		gmt_M_grd_loop (GMT, G, row, col, node) {
 			if (gmt_M_is_fnan (G->data[node]) || gmt_M_is_dnan (W->data[node]))
@@ -2709,8 +2712,9 @@ double gmt_grd_lmsscl (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID
 	uint64_t node, n = 0;
 	double wmode, lmsscl;
 	if (W) {	/* Weights provided */
-		unsigned int row, col;
+		openmp_int row, col;
 		struct GMT_OBSERVATION *pair = gmt_M_memory (GMT, NULL, G->header->nm, struct GMT_OBSERVATION);
+		if (pair == NULL) return 0.0;
 		if (mode) {	/* Already got the mode */
 			wmode = *mode;
 			/* 3. Compute the absolute deviations from this mode */
@@ -2773,7 +2777,8 @@ GMT_LOCAL void gmtstat_get_geo_cellarea (struct GMT_CTRL *GMT, struct GMT_GRID *
 	 * P.Wessel, July 2016.
 	 */
 	uint64_t node;
-	unsigned int row, col, j, first_row = 0, last_row = G->header->n_rows - 1, last_col = G->header->n_columns - 1, ltype;
+	openmp_int row, col, first_row = 0, last_row = (openmp_int)G->header->n_rows - 1, last_col = (openmp_int)G->header->n_columns - 1;
+	unsigned int j, ltype;
 	double lat, area, f, row_weight, col_weight = 1.0, R2 = pow (0.001 * GMT->current.proj.mean_radius, 2.0);	/* squared mean radius in km */
 	char *aux[6] = {"geodetic", "authalic", "conformal", "meridional", "geocentric", "parametric"};
 	char *rad[5] = {"mean (R_1)", "authalic (R_2)", "volumetric (R_3)", "meridional", "quadratic"};
@@ -2816,7 +2821,7 @@ GMT_LOCAL void gmtstat_get_geo_cellarea (struct GMT_CTRL *GMT, struct GMT_GRID *
 GMT_LOCAL void gmtstat_get_cart_cellarea (struct GMT_CTRL *GMT, struct GMT_GRID *G) {
 	/* Calculate Cartesian cell areas in user units */
 	uint64_t node;
-	unsigned int row, col, last_row = G->header->n_rows - 1, last_col = G->header->n_columns - 1;
+	openmp_int row, col, last_row = (openmp_int)G->header->n_rows - 1, last_col = (openmp_int)G->header->n_columns - 1;
 	double row_weight = 1.0, col_weight = 1.0, area = G->header->inc[GMT_X] * G->header->inc[GMT_Y];	/* All whole cells have same area */
 	gmt_M_unused(GMT);
 	gmt_M_row_loop (GMT, G, row) {	/* Loop over the rows */
