@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,14 +12,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * gmt_time.h contains definitions of structures dealing with time.
  *
  * Author:	Paul Wessel
  * Date:	01-OCT-2009
- * Version:	5 API
+ * Version:	6 API
  */
 
 /*!
@@ -28,8 +27,8 @@
  * \brief Definitions of structures dealing with time.
  */
 
-#ifndef _GMT_TIME_H
-#define _GMT_TIME_H
+#ifndef GMT_TIME_H
+#define GMT_TIME_H
 
 /*--------------------------------------------------------------------
  *			GMT TIME STRUCTURES
@@ -44,4 +43,4 @@ struct GMT_TIME_SYSTEM {
 	char unit;			/* User-defined time unit */
 };
 
-#endif  /* _GMT_TIME_H */
+#endif  /* GMT_TIME_H */
