@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -192,14 +192,17 @@ static int parse (struct GMT_CTRL *GMT, struct GMTBINSTATS_CTRL *Ctrl, struct GM
 		switch (opt->option) {
 
 			case '<':	/* Input file(s) */
-				if (GMT_Get_FilePath (API, GMT_IS_DATASET, GMT_IN, GMT_FILE_REMOTE, &(opt->arg))) n_errors++;;
+				if (GMT_Get_FilePath (API, GMT_IS_DATASET, GMT_IN, GMT_FILE_REMOTE, &(opt->arg))) n_errors++;
+				break;
+			case '>':	/* Output file  */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->G.active);
+				n_errors += gmt_get_required_file (GMT, opt->arg, opt->option, 0, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->G.file));
 				break;
 
 			/* Processes program-specific parameters */
 
 			case 'E':	/* NaN value */
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->E.active);
-				Ctrl->E.active = true;
 				if (opt->arg[0])
 					Ctrl->E.value = (opt->arg[0] == 'N' || opt->arg[0] == 'n') ? GMT->session.d_NaN : atof (opt->arg);
 				else {
@@ -209,18 +212,14 @@ static int parse (struct GMT_CTRL *GMT, struct GMTBINSTATS_CTRL *Ctrl, struct GM
 				break;
 			case 'G':	/* Output file */
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->G.active);
-				Ctrl->G.active = true;
-				if (opt->arg[0]) Ctrl->G.file = strdup (opt->arg);
-				if (GMT_Get_FilePath (API, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->G.file))) n_errors++;
+				n_errors += gmt_get_required_file (GMT, opt->arg, opt->option, 0, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->G.file));
 				break;
 			case 'I':	/* Grid spacings */
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->I.active);
-				Ctrl->I.active = true;
 				n_errors += gmt_parse_inc_option (GMT, 'I', opt->arg);
 				break;
 			case 'C':	/* -Ca|d|i|l|L|m|n|o|p|q[<val>]|r|s|u|U|z */
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->C.active);
-				Ctrl->C.active = true;
 				switch (opt->arg[0]) {
 					case 'a': Ctrl->C.mode = GMTBINSTATS_MEAN;	break;
 					case 'd': Ctrl->C.mode = GMTBINSTATS_MAD;	break;
@@ -248,16 +247,13 @@ static int parse (struct GMT_CTRL *GMT, struct GMTBINSTATS_CTRL *Ctrl, struct GM
 				break;
 			case 'N':	/* Normalize by area */
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->N.active);
-				Ctrl->N.active = true;
 				break;
 			case 'S':	/* Search radius */
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->S.active);
-				Ctrl->S.active = true;
 				Ctrl->S.mode = gmt_get_distance (GMT, opt->arg, &(Ctrl->S.radius), &(Ctrl->S.unit));
 				break;
 			case 'T':	/* Select hexagonal or rectangular tiling */
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->T.active);
-				Ctrl->T.active = true;
 				switch (opt->arg[0]) {
 					case 'h': Ctrl->T.mode = GMTBINSTATS_HEXAGONAL; break;
 					case 'r': Ctrl->T.mode = GMTBINSTATS_RECTANGULAR; break;
@@ -269,12 +265,11 @@ static int parse (struct GMT_CTRL *GMT, struct GMTBINSTATS_CTRL *Ctrl, struct GM
 				break;
 			case 'W':	/* Use weights */
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->W.active);
-				Ctrl->W.active = true;
 				if (gmt_validate_modifiers (GMT, opt->arg, 'W', "sw", GMT_MSG_ERROR)) n_errors++;
 				Ctrl->W.sigma = (strstr (opt->arg, "+s")) ? true : false;
 				break;
 			default:	/* Report bad options */
-				n_errors += gmt_default_error (GMT, opt->option);
+				n_errors += gmt_default_option_error (GMT, opt);
 				break;
 		}
 	}
@@ -449,9 +444,11 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 
 	bool wrap_180, replicate_x, replicate_y, hex_tiling, rect_tiling, geographic;
 
-	int col_0, row_0, row, col, row_end, col_stop, col_start, ii, jj, error = 0;
-	unsigned int n_cols, k, rowu, colu, d_row, y_wrap_kk, y_wrap_ij, max_d_col, x_wrap;
-	unsigned int *d_col = NULL, *n_in_circle = NULL, *n_alloc = NULL;
+	openmp_int rowu, colu, max_d_col, d_row, *d_col = NULL;
+	int row, col, col_0, row_0, row_end, col_stop, col_start;
+	int ii, jj, error = 0;
+	unsigned int n_cols, k, y_wrap_kk, y_wrap_ij, x_wrap;
+	unsigned int *n_in_circle = NULL, *n_alloc = NULL;
 	unsigned int gmt_mode_selection = 0, GMT_n_multiples = 0, col_inc = 1;
 
 	uint64_t ij, kk, n = 0, n_read = 0;
@@ -528,7 +525,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 		GMT_GRID_DEFAULT_REG, GMT_NOTSET, NULL)) == NULL) Return (API->error);
 	HH = gmt_get_H_hidden (Grid->header);
 
-	gmt_M_grd_loop (GMT, Grid, row, col, ij)	/* Initialize grid to NaN which is our flag for an unused grid node */
+	gmt_M_grd_loop (GMT, Grid, rowu, colu, ij)	/* Initialize grid to NaN which is our flag for an unused grid node */
 		Grid->data[ij] = GMT->session.d_NaN;
 
 	/* Initialize the input since we are doing record-by-record reading */
@@ -648,8 +645,8 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 
 		/* Loop over all nodes within radius of this node */
 
-		row_end = row_0 + d_row;
-		for (row = row_0 - d_row; row <= row_end; row++) {
+		row_end = row_0 + (int)d_row;
+		for (row = row_0 - (int)d_row; row <= row_end; row++) {
 
 			jj = row;
 			if (gmt_y_out_of_bounds (GMT, &jj, Grid->header, &wrap_180)) continue;	/* Outside y-range.  This call must happen BEFORE gmt_x_out_of_bounds as it sets wrap_180 */
@@ -659,8 +656,8 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 				dy = in[GMT_Y] - y0[rowu];
 				if (fabs (dy) > dy2) continue;	/* Outside regardless of x-coordinate */
 			}
-			col_stop  = col_0 + d_col[jj];
-			col_start = col_0 - d_col[jj];
+			col_stop  = col_0 + (int)d_col[jj];
+			col_start = col_0 - (int)d_col[jj];
 			if (hex_tiling) {	/* Make sure we start and stop on valid pseudo-columns so both col,row are even or odd */
 				if (skip_column (row,col_start)) col_start++;
 				if (skip_column (row,col_stop))  col_stop--;
@@ -716,13 +713,13 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 				if (replicate_x) {	/* Must check if we have to replicate a column */
 					if (colu == 0) 	/* Must replicate left to right column */
 						gmtbinstats_assign_node (GMT, Grid, visited, w, s, zp, zw_pair, n_in_circle, n_alloc, ij+x_wrap, kk+x_wrap, Ctrl->C.mode, in, weight);
-					else if (colu == HH->nxp)	/* Must replicate right to left column */
+					else if (colu == (openmp_int)HH->nxp)	/* Must replicate right to left column */
 						gmtbinstats_assign_node (GMT, Grid, visited, w, s, zp, zw_pair, n_in_circle, n_alloc, ij-x_wrap, kk-x_wrap, Ctrl->C.mode, in, weight);
 				}
 				if (replicate_y) {	/* Must check if we have to replicate a row */
 					if (rowu == 0)	/* Must replicate top to bottom row */
 						gmtbinstats_assign_node (GMT, Grid, visited, w, s, zp, zw_pair, n_in_circle, n_alloc, ij+y_wrap_ij, kk+y_wrap_kk, Ctrl->C.mode, in, weight);
-					else if (rowu == HH->nyp)	/* Must replicate bottom to top row */
+					else if (rowu == (openmp_int)HH->nyp)	/* Must replicate bottom to top row */
 						gmtbinstats_assign_node (GMT, Grid, visited, w, s, zp, zw_pair, n_in_circle, n_alloc, ij-y_wrap_ij, kk-y_wrap_kk, Ctrl->C.mode, in, weight);
 				}
 			}
@@ -739,16 +736,16 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 	kk = 0;
 	switch (Ctrl->C.mode) {
 		case GMTBINSTATS_MEAN: case GMTBINSTATS_MEANW:	/* Compute [weighted] mean */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (w[kk] > 0.0) Grid->data[ij] /= w[kk], n++;
 				kk++;
 			}
 			break;
 		case GMTBINSTATS_MEDIAN:	/* Compute plain medians */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if ((k = n_in_circle[kk])) {
 					gmt_sort_array (GMT, zp[kk], k, GMT_FLOAT);
-					Grid->data[ij] = (k%2) ? zp[kk][k/2] : 0.5 * (zp[kk][(k-1)/2] + zp[kk][row/2]);
+					Grid->data[ij] = (k%2) ? zp[kk][k/2] : 0.5 * (zp[kk][(k-1)/2] + zp[kk][rowu/2]);
 					gmt_M_free (GMT, zp[kk]);
 					n++;
 				}
@@ -756,7 +753,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_MEDIANW:	/* Compute weighted medians */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (n_in_circle[kk]) {
 					Grid->data[ij] = (gmt_grdfloat)gmt_median_weighted (GMT, zw_pair[kk], n_in_circle[kk]);
 					gmt_M_free (GMT, zw_pair[kk]);
@@ -766,7 +763,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_MODE:	/* Compute plain modes */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if ((k = n_in_circle[kk])) {
 					gmt_mode_f (GMT, zp[kk], k, k/2, true, gmt_mode_selection, &GMT_n_multiples, &zmode);
 					Grid->data[ij] = (gmt_grdfloat)zmode;
@@ -777,7 +774,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_MODEW:	/* Compute weighted modes */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (n_in_circle[kk]) {
 					Grid->data[ij] = (gmt_grdfloat)gmt_mode_weighted (GMT, zw_pair[kk], n_in_circle[kk]);
 					gmt_M_free (GMT, zw_pair[kk]);
@@ -787,7 +784,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_QUANT:	/* Compute plain quantile */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if ((k = n_in_circle[kk])) {
 					Grid->data[ij] = (gmt_grdfloat) gmt_quantile_f (GMT, zp[kk], Ctrl->C.quant, k);
 					gmt_M_free (GMT, zp[kk]);
@@ -797,7 +794,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_QUANTW:	/* Compute weighted quantile */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (n_in_circle[kk]) {
 					Grid->data[ij] = (gmt_grdfloat)gmt_quantile_weighted (GMT, zw_pair[kk], n_in_circle[kk], 0.01 * Ctrl->C.quant);
 					gmt_M_free (GMT, zw_pair[kk]);
@@ -807,7 +804,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_IRANGE:	/* Compute plain inter quartile range */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if ((k = n_in_circle[kk])) {
 					Grid->data[ij] = (gmt_grdfloat) (gmt_quantile_f (GMT, zp[kk], 75.0, k) - gmt_quantile_f (GMT, zp[kk], 25.0, k));
 					gmt_M_free (GMT, zp[kk]);
@@ -817,7 +814,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_IRANGEW:	/* Compute weighted quantile */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if ((k = n_in_circle[kk])) {
 					Grid->data[ij] = (gmt_grdfloat) (gmt_quantile_weighted (GMT, zw_pair[kk], k, 0.75) - gmt_quantile_weighted (GMT, zw_pair[kk], k, 0.25));
 					gmt_M_free (GMT, zw_pair[kk]);
@@ -827,12 +824,12 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_COUNT: case GMTBINSTATS_COUNTW: case GMTBINSTATS_SUM:	/* Compute count or sum */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (!gmt_M_is_fnan (Grid->data[ij]) && Grid->data[ij] > 0.0) n++;
 			}
 			break;
 		case GMTBINSTATS_ZRANGE:	/* Compute full data range */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (visited[kk]) {
 					Grid->data[ij] -= s[kk];
 					n++;
@@ -841,7 +838,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_STD:	/* Compute plain standard deviation */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (n_in_circle[kk] > 1) {
 					Grid->data[ij] = sqrt ((n_in_circle[kk] * s[kk] - Grid->data[ij] * Grid->data[ij]) / (n_in_circle[kk] * (n_in_circle[kk] - 1)));
 					n++;
@@ -852,7 +849,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_STDW:	/* Compute weighted standard deviation */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (n_in_circle[kk] > 1) {
 					Grid->data[ij] = sqrt ((w[kk] * s[kk] - Grid->data[ij] * Grid->data[ij]) / ((n_in_circle[kk] - 1) * w[kk] * w[kk] /n_in_circle[kk]));
 					n++;
@@ -863,7 +860,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_RMS:	/* Compute plain rms */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (n_in_circle[kk]) {
 					Grid->data[ij] = sqrt (Grid->data[ij] / n_in_circle[kk]);
 					n++;
@@ -872,7 +869,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_RMSW:	/* Compute weighted rms */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (w[kk] > 0.0) {
 					Grid->data[ij] = sqrt (Grid->data[ij] / w[kk]);
 					n++;
@@ -881,10 +878,10 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_MAD:	/* Compute plain MAD */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if ((k = n_in_circle[kk])) {
 					gmt_sort_array (GMT, zp[kk], k, GMT_FLOAT);
-					median = (k%2) ? zp[kk][k/2] : 0.5 * (zp[kk][(k-1)/2] + zp[kk][row/2]);
+					median = (k%2) ? zp[kk][k/2] : 0.5 * (zp[kk][(k-1)/2] + zp[kk][rowu/2]);
 					gmt_getmad_f (GMT, zp[kk], k, median, &mad);
 					Grid->data[ij] = mad;
 					gmt_M_free (GMT, zp[kk]);
@@ -894,7 +891,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_MADW:	/* Compute weighted MAD */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (n_in_circle[kk]) {
 					median = gmt_median_weighted (GMT, zw_pair[kk], n_in_circle[kk]);
 					/* Compute the absolute deviations from this median */
@@ -908,7 +905,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_LMSSCL:	/* Compute plain mode lmsscale */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if ((k = n_in_circle[kk])) {
 					gmt_mode_f (GMT, zp[kk], k, k/2, true, gmt_mode_selection, &GMT_n_multiples, &zmode);
 					gmt_getmad_f (GMT, zp[kk], k, zmode, &mad);
@@ -920,7 +917,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			}
 			break;
 		case GMTBINSTATS_LMSSCLW:	/* Compute weighted modes */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij) {
 				if (n_in_circle[kk]) {
 					zmode = gmt_mode_weighted (GMT, zw_pair[kk], n_in_circle[kk]);
 					for (k = 0; k < n_in_circle[kk]; k++) zw_pair[kk][k].value = (gmt_grdfloat)fabs (zw_pair[kk][k].value - zmode);
@@ -939,7 +936,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 
 	if (Ctrl->N.active) {	/* Area normalization */
 		double scale = 1.0 / Ctrl->S.area;
-		gmt_M_grd_loop (GMT, Grid, row, col, ij)
+		gmt_M_grd_loop (GMT, Grid, rowu, colu, ij)
 			Grid->data[ij] *= scale;
 	}
 
@@ -964,12 +961,12 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 			Return (API->error);
 		}
 		S = D->table[0]->segment[0];	/* Only a single segment will be written */
-		gmt_M_row_loop (GMT, Grid, row) {
-			gmt_M_col_loop (GMT, Grid, row, col, ij) {
-				if (skip_column(row,col)) continue;	/* Unused pseudo=grid node */
+		gmt_M_row_loop (GMT, Grid, rowu) {
+			gmt_M_col_loop (GMT, Grid, rowu, colu, ij) {
+				if (skip_column(rowu,colu)) continue;	/* Unused pseudo=grid node */
 				if (gmt_M_is_fnan (Grid->data[ij])) continue;	/* Unvisited node */
-				S->data[GMT_X][k] = x0[col];
-				S->data[GMT_Y][k] = y0[row];
+				S->data[GMT_X][k] = x0[colu];
+				S->data[GMT_Y][k] = y0[rowu];
 				S->data[GMT_Z][k] = Grid->data[ij];
 				k++;
 			}
@@ -980,7 +977,7 @@ EXTERN_MSC int GMT_gmtbinstats (void *V_API, int mode, void *args) {
 	}
 	else {	/* Write a grid */
 		if (!gmt_M_is_fnan (Ctrl->E.value)) {	/* Replace NaNs with the -E value */
-			gmt_M_grd_loop (GMT, Grid, row, col, ij)
+			gmt_M_grd_loop (GMT, Grid, rowu, colu, ij)
 				if (gmt_M_is_fnan (Grid->data[ij])) Grid->data[ij] = Ctrl->E.value;
 		}
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Grid)) {
