@@ -1,7 +1,7 @@
 /*
  * Include file defining macros, functions and structures used in gshhg.c
  *
- *	Copyright (c) 1996-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1996-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 
 /*!
  * \file gmt_gshhg.h
- * \brief Include file defining macros, functions and structures used in gshhg.c 
+ * \brief Include file defining macros, functions and structures used in gshhg.c
  */
 
 #ifndef _GMT_GSHHG
@@ -34,7 +34,7 @@
 
 #include "gmt_notposix.h"
 
-#include "common_byteswap.h"
+#include "gmt_common_byteswap.h"
 
 #include "gshhg.h"		/* The definition of the GSHHG header structure */
 
