@@ -1,8 +1,18 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77header.c 19521 2017-12-27 22:38:24Z pwessel $
  *
- *    Copyright (c) 2004-2018 by P. Wessel and Michael Chandler
- *    See README file for copying and redistribution conditions.
+ *	Copyright (c) 2004-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	See LICENSE.TXT file for copying and redistribution conditions.
+ *
+ *	This program is free software; you can redistribute it and/or modify
+ *	it under the terms of the GNU Lesser General Public License as published by
+ *	the Free Software Foundation; version 3 or any later version.
+ *
+ *	This program is distributed in the hope that it will be useful,
+ *	but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *	GNU Lesser General Public License for more details.
+ *
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * mgd77header.c reads an NGDC A77 file, determines temporal and spatial extents,
@@ -14,7 +24,7 @@
  * Note: Program expects first row to be a header as (no header row will trigger error):
  * #rec	TZ	year	month	day	hour	min	lat		lon		ptc	twt	depth	bcc	btc	mtf1	mtf2	mag	msens	diur	msd	gobs	eot	faa	nqc	id	sln	sspn
  *
- * Author:	Michael Chandler, ported to GMT5 by P. Wessel
+ * Author:	Michael Hamilton (nee Chandler), ported to GMT5 by P. Wessel
  * Date:	23-MAY-2012
  * Prted to GMT5 on 13-DEC-2016 by P. Wessel
  *
@@ -23,7 +33,8 @@
 #include "gmt_dev.h"
 #include "mgd77.h"
 
-#define THIS_MODULE_NAME	"mgd77header"
+#define THIS_MODULE_CLASSIC_NAME	"mgd77header"
+#define THIS_MODULE_MODERN_NAME	"mgd77header"
 #define THIS_MODULE_LIB		"mgd77"
 #define THIS_MODULE_PURPOSE	"Create MGD77 headers from A77 files"
 #define THIS_MODULE_KEYS	""
@@ -40,64 +51,69 @@ EXTERN_MSC int MGD77_Read_File_nohdr (struct GMT_CTRL *GMT, char *file, struct M
 
 struct MGD77HEADER_CTRL {	/* All control options for this program (except common args) */
 	/* active is true if the option has been activated */
-	struct H {	/* -H<file> */
+	struct MGD77HEADER_H {	/* -H<file> */
 		bool active;
 		char *file;
 	} H;
-	struct M {	/* -Mf[<item>|r|t] */
+	struct MGD77HEADER_M {	/* -Mf[<item>|r|t] */
 		bool active;
 		unsigned int mode;
 		unsigned int flag;
 	} M;
 };
 
-GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
+static void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct MGD77HEADER_CTRL *C = NULL;
-	
+
 	C = gmt_M_memory (GMT, NULL, 1, struct MGD77HEADER_CTRL);
-	
+
 	/* Initialize values whose defaults are not 0/false/NULL */
-	
+
 	C->M.mode = 1;
 	return (C);
 }
 
-GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct MGD77HEADER_CTRL *C) {	/* Deallocate control structure */
+static void Free_Ctrl (struct GMT_CTRL *GMT, struct MGD77HEADER_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
 	gmt_M_str_free (C->H.file);
-	gmt_M_free (GMT, C);	
+	gmt_M_free (GMT, C);
 }
 
-GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
+static int usage (struct GMTAPI_CTRL *API, int level) {
 	struct MGD77_CONTROL M;
 
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: mgd77info <cruise(s)>  [-H<headinfo>] [-Mf[<item>]|r|e|h] [%s]\n\n", GMT_V_OPT);
-        
+	GMT_Usage (API, 0, "usage: %s <cruise(s)>  [-H<headertable>] [-Mf[<item>]|r|t] [%s] [%s]\n", name, GMT_V_OPT, GMT_PAR_OPT);
+
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
-             
+
 	MGD77_Init (API->GMT, &M);		/* Initialize MGD77 Machinery */
+	GMT_Message (API, GMT_TIME_NONE, "  REQUIRED ARGUMENTS:\n");
 	MGD77_Cruise_Explain (API->GMT);
-	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-H Read and assign header values from a file. Each input file row gives an exact\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   header_field_name, space or tab, and header value. Values are read according to\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   NGDC's MGD77 header format specification.\n\t\te.g.,\n\t\tSource_Institution Univ. of Hawaii\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t\tPort_of_Arrival Honolulu, HAWAII\n\t\t...\n	   See mgd77info -Mf output for recognized header field names.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-M Print header items.  Append type of presentation:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     f: Print header items individually, one per line.  Append name of a particular\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        item (e.g., Port_of_Departure), all [Default], or - to see a list of items.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        You can also use the number of the item.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     r: Display raw original MGD77 header records [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     t: Display raw original M77T header records.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\n  OPTIONAL ARGUMENTS:\n");
+	GMT_Usage (API, 1, "\n-H<headertable>");
+	GMT_Usage (API, -2, "Read and assign header values from the given file. Each input file row gives an exact "
+		"header_field_name, space or tab, and header value. Values are read according to "
+		"NGDC's MGD77 header format specification. Two examples are given:");
+	GMT_Usage (API, 3, "Source_Institution Univ. of Hawaii.");
+	GMT_Usage (API, 3, "Port_of_Arrival Honolulu, HAWAII");
+	GMT_Usage (API, -2, "Note: See mgd77info -Mf output for recognized header field names.");
+	GMT_Usage (API, 1, "\n-Mf[<item>]|r|t");
+	GMT_Usage (API, -2, "Print header items.  Append type of presentation:");
+	GMT_Usage (API, 3, "f: Print header items individually, one per line.  Append name of a particular "
+		"item (e.g., Port_of_Departure), all [Default], or - to see a list of items."
+		"You can also use the number of the item.");
+	GMT_Usage (API, 3, "r: Display raw original MGD77 header records [Default].");
+	GMT_Usage (API, 3, "t: Display raw original M77T header records.");
 	GMT_Option (API, "V,.");
-	
+
 	MGD77_end (API->GMT, &M);	/* Close machinery */
 
 	return (GMT_MODULE_USAGE);
 }
 
-GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77HEADER_CTRL *Ctrl, struct GMT_OPTION *options, struct MGD77_CONTROL *M) {
+static int parse (struct GMT_CTRL *GMT, struct MGD77HEADER_CTRL *Ctrl, struct GMT_OPTION *options, struct MGD77_CONTROL *M) {
 	/* This parses the options provided to mgd77info and sets parameters in CTRL.
 	 * Any GMT common options will override values set previously by other commands.
 	 * It also replaces any file names specified as input or output with the data ID
@@ -119,12 +135,12 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77HEADER_CTRL *Ctrl, struct
 			/* Processes program-specific parameters */
 
 			case 'H':
-				Ctrl->H.active = true;
-				Ctrl->H.file = strdup (opt->arg);
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->H.active);
+				n_errors += gmt_get_required_file (GMT, opt->arg, opt->option, 0, GMT_IS_DATASET, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->H.file));
 				break;
-				
+
 			case 'M':
-				Ctrl->M.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->M.active);
 				if (opt->arg[0] == 'f') {
 					Ctrl->M.mode = FORMATTED_HEADER;
 					sval = MGD77_Select_Header_Item (GMT, M, &opt->arg[1]);
@@ -136,13 +152,13 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77HEADER_CTRL *Ctrl, struct
 				else if (opt->arg[0] == 't')
 					Ctrl->M.mode = M77T_HEADER;
 				else {
-					GMT_Report (API, GMT_MSG_NORMAL, "Option -M Bad modifier (%c). Use -Mf|r|t!\n", opt->arg[0]);
+					GMT_Report (API, GMT_MSG_ERROR, "Option -M Bad modifier (%c). Use -Mf|r|t!\n", opt->arg[0]);
 					n_errors++;
 				}
 				break;
 
 			default:	/* Report bad options */
-				n_errors += gmt_default_error (GMT, opt->option);
+				n_errors += gmt_default_option_error (GMT, opt);
 				break;
 		}
 	}
@@ -157,18 +173,18 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77HEADER_CTRL *Ctrl, struct
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
-int GMT_mgd77header (void *V_API, int mode, void *args) {
+EXTERN_MSC int GMT_mgd77header (void *V_API, int mode, void *args) {
 	int i, id, argno, length, id_col, t_col, x_col, y_col, saved_range, use;
 	int n_paths, counter[MGD77_MAX_COLS], quad_no, n_quad;
 	int b_col, twt_col, g_col, m_col, f_col, mt1_col, mt2_col;
 	int tendeg[36][18], tenx, teny, nten = 0, tquad;
-	
+
 	uint64_t rec;
 
 	bool error = false;
 	bool quad[4] = {false, false, false, false};
 
-	double this_dist, this_lon, this_lat, last_lon, last_lat, dx, dy, dlon, ds, lon_w;
+	double this_lon, this_lat, last_lon, last_lat, dx, dy, dlon, ds, lon_w;
 	double xmin, xmax, xmin1, xmin2, xmax1, xmax2, ymin, ymax, this_time, tmin, tmax;
 	double *dvalue[MGD77_MAX_COLS];
 
@@ -194,12 +210,11 @@ int GMT_mgd77header (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, NULL, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	MGD77_Init (GMT, &M);		/* Initialize input MGD77 Machinery */
@@ -212,7 +227,7 @@ int GMT_mgd77header (void *V_API, int mode, void *args) {
 	tod = localtime (&tt);
 	if (Ctrl->H.active) {
 		if ((fp = gmt_fopen (GMT, Ctrl->H.file, "r")) == NULL) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Error: Cannot open header items input file (%s)\n", Ctrl->H.file);
+			GMT_Report (API, GMT_MSG_ERROR, "Cannot open header items input file (%s)\n", Ctrl->H.file);
 			Return (GMT_NO_INPUT);
 		}
 	}
@@ -224,14 +239,15 @@ int GMT_mgd77header (void *V_API, int mode, void *args) {
 	n_paths = MGD77_Path_Expand (GMT, &M, options, &list);	/* Get list of requested IDs */
 
 	if (n_paths <= 0) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Error: No cruises given\n");
+		GMT_Report (API, GMT_MSG_ERROR, "No cruises given\n");
 		if (fp) gmt_fclose (GMT, fp);
+		MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 		Return (GMT_NO_INPUT);
 	}
 
 	saved_range = GMT->current.io.geo.range;	/* We may have to reset thisso keep a copy */
 	gmt_set_geographic (GMT, GMT_OUT);	/* Output lon/lat */
-	GMT->current.io.col_type[GMT_OUT][2] = M.time_format;
+	gmt_set_column_type (GMT, GMT_OUT, GMT_Z, M.time_format);
 
 	use = (M.original || M.format != MGD77_FORMAT_CDF) ? MGD77_ORIG : MGD77_REVISED;
 
@@ -239,12 +255,14 @@ int GMT_mgd77header (void *V_API, int mode, void *args) {
 
 		if (MGD77_Open_File (GMT, list[argno], &M, MGD77_READ_MODE)) continue;
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Now processing cruise %s\n", list[argno]);
+		GMT_Report (API, GMT_MSG_INFORMATION, "Now processing cruise %s\n", list[argno]);
 
 		D = MGD77_Create_Dataset (GMT);
 
 		if (MGD77_Read_File_nohdr (GMT, list[argno], &M, D)) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Error reading data for cruise %s\n", list[argno]);
+			GMT_Report (API, GMT_MSG_ERROR, "Failure while reading data for cruise %s\n", list[argno]);
+			MGD77_Free_Dataset (GMT, &D);	/* Free memory allocated by MGD77_Read_File */
+			MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 			Return (GMT_DATA_READ_ERROR);
 		}
 
@@ -261,7 +279,7 @@ int GMT_mgd77header (void *V_API, int mode, void *args) {
 		id_col = MGD77_Get_Column	(GMT, "id", &M);
 
 		tmin = tmax = GMT->session.d_NaN;
-		this_dist = this_lon = this_lat = ds = this_time = 0.0;
+		this_lon = this_lat = ds = this_time = 0.0;
 		xmin1 = xmin2 = 360.0;
 		xmax1 = xmax2 = -360.0;
 		ymin = 180.0;
@@ -313,7 +331,6 @@ int GMT_mgd77header (void *V_API, int mode, void *args) {
 				dx = dlon * cosd (0.5 * (this_lat + last_lat));
 				dy = this_lat - last_lat;
 				ds = GMT->current.proj.DIST_KM_PR_DEG * hypot (dx, dy);
-				this_dist += ds;
 			}
 			ymin = MIN (this_lat, ymin);
 			ymax = MAX (this_lat, ymax);
@@ -377,7 +394,9 @@ int GMT_mgd77header (void *V_API, int mode, void *args) {
 	        if (xmax > 180) xmax -= 360.0;
 
 		if (gmt_M_is_dnan(tmin) || gmt_M_is_dnan(tmax)) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Abort: cruise %s no time records\n", M.NGDC_id);
+			GMT_Report (API, GMT_MSG_ERROR, "Abort: cruise %s no time records\n", M.NGDC_id);
+			MGD77_Free_Dataset (GMT, &D);
+			MGD77_Close_File (GMT, &M);
 			Return (GMT_DATA_READ_ERROR);
 		}
 		MGD77_Close_File (GMT, &M);
@@ -420,12 +439,12 @@ int GMT_mgd77header (void *V_API, int mode, void *args) {
 						if (teny >= 9)
 							tquad = 7;
 					}
-					if (teny > 8) sprintf (value,"%s%.1d%01.0f%02.0f,",value,tquad,fabs(teny-9.0),fabs(tenx-18.0));
-					else sprintf (value,"%s%.1d%01.0f%02.0f,",value,tquad,fabs(teny-8.0),fabs(tenx-18.0));
+					if (teny > 8) snprintf (value, BUFSIZ, "%s%.1d%01.0f%02.0f,",value,tquad,fabs(teny-9.0),fabs(tenx-18.0));
+					else snprintf (value, BUFSIZ, "%s%.1d%01.0f%02.0f,",value,tquad,fabs(teny-8.0),fabs(tenx-18.0));
 				}
 			}
 		}
-		sprintf (value, "%s9999", value); i++;
+		snprintf (value, BUFSIZ, "%s9999", value); i++;
 		while (i < 30) { /* MGD77 format can store up to this many 10x10 identifiers */
  			sprintf (value,"%s,   0",value);
 			i++;
