@@ -1,13 +1,18 @@
 /*--------------------------------------------------------------------
  *
- *    Copyright (c) 2005-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
- * mgd77manage is used to (1) remove data columns from mgd77+ files
- * or (2) add a new data column to mgd77+ files.  Data can be added
- * from data tables, created from reference field formulas, or by
- * sampling grids along track.  Data from tables may be numbers or
- * text strings.
+ *	Copyright (c) 2005-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
- *    See README file for copying and redistribution conditions.
+ *	This program is free software; you can redistribute it and/or modify
+ *	it under the terms of the GNU Lesser General Public License as published by
+ *	the Free Software Foundation; version 3 or any later version.
+ *
+ *	This program is distributed in the hope that it will be useful,
+ *	but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *	GNU Lesser General Public License for more details.
+ *
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  *
@@ -122,73 +127,83 @@ static void Free_Ctrl (struct GMT_CTRL *GMT, struct MGD77MANAGE_CTRL *C) {	/* De
 static int usage (struct GMTAPI_CTRL *API, int level) {
 	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: %s <cruise(s)> [-Aa|c|d|D|e|E|g|i|n|t|T<info>[+f]] [-D<name1>,<name2>,...]\n", name);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-E<no_char>] [-F] [-I<abbrev>/<name>/<units>/<size>/<scale>/<offset>/\"comment\"]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-N%s] [%s] [%s] [%s]\n\t[%s] [%s] [%s]\n\n",
-	             GMT_LEN_UNITS2_DISPLAY, GMT_Rgeo_OPT, GMT_V_OPT, GMT_bi_OPT, GMT_j_OPT, GMT_n_OPT, GMT_PAR_OPT);
+	GMT_Usage (API, 0, "usage: %s <cruise(s)> [-Aa|c|d|D|e|E|g|i|n|t|T<info>[+f]] [-D<name1>,<name2>,...] "
+		"[-E<no_char>] [-F] [-I<abbrev>/<name>/<units>/<size>/<scale>/<offset>/\"comment\"] "
+		"[-N%s] [%s] [%s] [%s] [%s] [%s] [%s]\n",
+	    name, GMT_LEN_UNITS2_DISPLAY, GMT_Rgeo_OPT, GMT_V_OPT, GMT_bi_OPT, GMT_j_OPT, GMT_n_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
+	GMT_Message (API, GMT_TIME_NONE, "  REQUIRED ARGUMENTS:\n");
 	MGD77_Cruise_Explain (API->GMT);
-	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-A Append a new data column to the given files.  Append +f to overwrite an\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   existing column with same name with new data [Default will refuse if an\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   existing column has the same abbreviation as the new data].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   The code letters are:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   a: Give filename with a new column to add.  We expect a single-column file\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      with the same number of records as the MGD77 file.  Only one cruise can be set.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      If filename is - we read from stdin.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   c: Create a new column to be calculated from existing columns.  Add code:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        m = IGRF total field, c = Carter correction, g = IGF (\"normal gravity\").\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        r = recomputed magnetic anomaly rmag = mtfx - IGRF total field.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        Append x for which mtfx field to use (1 or 2) [1].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        For g, optionally append 1-4 to select the gravity formula to use:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        1 = Heiskanen 1924, 2 = International 1930, 3 = IGF1967, 4 = IGF1980.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        [Default uses formula specified in the MGD77 header, or 4 if not valid].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   d: Give filename with (dist [see -N], data) for a new column.  We expect a two-column file\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      with distances (in km) in first column and data values in 2nd.  Only one cruise can be set.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      If filename is - we read from stdin.  Only records with matching distance will have data assigned.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   D: Same as d but we interpolate between the dist,data pairs to fill in all data records.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   e: Ingest MGD77 error/correction information (e77) produced by mgd77sniffer.  We will look\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      for the <cruise>.e77 file in the current directory or in $MGD77_HOME/E77.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      By default we will apply recommended header (h) and systematic fixes (f) and set all data bit flags.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      Append a combination of these flags to change the default accordingly:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        h = Ignore all header recommendations\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        f = Ignore all systematic fixes recommendations\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        n = Ignore data record bitflags pertaining to navigation (time, lon, lat).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        v = Ignore data record bitflags pertaining to data values.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        s = Ignore data record bitflags pertaining to data slopes (gradients).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      Use -DE to ignore the verification status of the e77 file [Default requires verification to be Y].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      Note: Previous E77 information will be removed prior to processing this E77 information.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   g: Sample a GMT grid along track. (also see -n; use -R to select a sub-region).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      Append filename of the GMT grid.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   i: Sample a Sandwell/Smith *.img Mercator grid along track (also see -n; use -R to select a sub-region).\n");
-	gmt_img_syntax (API->GMT);
-	GMT_Message (API, GMT_TIME_NONE, "\t   n: Give filename with (rec_no, data) for a new column.  We expect a two-column file\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      with record numbers (0 means 1st row) in first column and data values in 2nd.  Only one cruise can be set.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      If filename is - we read from stdin.  Only records with matching record numbers will have data assigned.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   t: Give filename with (abstime, data) for a new column.  We expect a two-column file\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      with dateTclock strings in first column and data values in 2nd.  Only one cruise can be set.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      If filename is - we read from stdin.  Only records with matching times will have data assigned.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   T: Same as t but we interpolate between the time, data pairs to fill in all data records.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-D Delete the columns listed from all the cruise data files.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   The columns are removed before any data are added.  It is not a substitute for -A...+f.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   However, sometimes the shape of new data demands the old to be deleted first (you will be told).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-E Give character used to fill empty/missing string columns [9]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-F Force mode.  This allows you to even replace the standard MGD77 columns [only extended columns can be changed].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-I In addition to the file information above, you must also specify column information:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      abbrev:  Short, abbreviated word (lower case only), like satfaa (%d char max).\n", MGD77_COL_ABBREV_LEN);
-	GMT_Message (API, GMT_TIME_NONE, "\t      name:    Descriptive name, like \"Geosat/ERS-1 Free-air gravity\" (%d char max).\n", MGD77_COL_NAME_LEN);
-	GMT_Message (API, GMT_TIME_NONE, "\t      units:   Units for the column (e.g., mGal, gamma, km) (%d char max).\n", MGD77_COL_NAME_LEN);
-	GMT_Message (API, GMT_TIME_NONE, "\t      size:    Either t(ext), b(yte), s(hort), f(loat), i(nt), or d(ouble).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      scale:   Multiply data by this scale before writing to mgd77+ file.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      offset:  Add after scaling before writing to mgd77+ file.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      comment: Any text (in double quotes) for information about column (%d char max).\n", MGD77_COL_COMMENT_LEN);
-	GMT_Message (API, GMT_TIME_NONE, "\t      -I is ignored by -Ae.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Note for text: Interpolation is not allowed, and \"not-a-string\" is created from -E.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-N Append your choice for distance unit (if -Ad|D are set). Choose among:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   m(e)ter, (f)oot, (k)m, (M)ile, (n)autical mile, or s(u)rvey foot [Default is -Nk].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t    See -j for selecting distance calculation procedure.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\n  OPTIONAL ARGUMENTS:\n");
+	GMT_Usage (API, 1, "\n-Aa|c|d|D|e|E|g|i|n|t|T<info>[+f]");
+	GMT_Usage (API, -2, "Append a new data column to the given files.  Append +f to overwrite an "
+		"existing column with same name with new data [Default will refuse if an "
+		"existing column has the same abbreviation as the new data]. "
+		"The code letters are:");
+	GMT_Usage (API, 3, "a: Give filename with a new column to add.  We expect a single-column file "
+		"with the same number of records as the MGD77 file. Only one cruise can be set. "
+		"If filename is - we read from standard input.");
+	GMT_Usage (API, 3, "c: Create a new column to be calculated from existing columns.  Add a code:");
+	GMT_Usage (API, 4, "m: IGRF total field.");
+	GMT_Usage (API, 4, "c: Carter correction.");
+	GMT_Usage (API, 4, "g: IGF (\"normal gravity\"). Optionally append 1-4 to select the gravity formula to use:");
+	GMT_Usage (API, 5, "1: Heiskanen 1924.");
+	GMT_Usage (API, 5, "2: International 1930.");
+	GMT_Usage (API, 5, "3: IGF1967.");
+	GMT_Usage (API, 5, "4: IGF1980.");
+	GMT_Usage (API, -4, "[Default uses formula specified in the MGD77 header, or 4 if not valid].");
+	GMT_Usage (API, 4, "r: Recomputed magnetic anomaly rmag = mtfx - IGRF total field. "
+		"Append x for which mtfx field to use (1 or 2) [1].");
+	GMT_Usage (API, 3, "d: Give filename with (dist [see -N], data) for a new column.  We expect a two-column file "
+		"with distances (in km) in first column and data values in 2nd.  Only one cruise can be set. "
+		"If filename is - we read from standard input.  Only records with matching distance will have data assigned.");
+	GMT_Usage (API, 3, "D: Same as d but we interpolate between the dist,data pairs to fill in all data records.");
+	GMT_Usage (API, 3, "e: Ingest MGD77 error/correction information (e77) produced by mgd77sniffer.  We will look "
+		"for the <cruise>.e77 file in the current directory or in $MGD77_HOME/E77. "
+		"By default we will apply recommended header (h) and systematic fixes (f) and set all data bit flags. "
+		"Append a combination of these flags to change the default accordingly:");
+	GMT_Usage (API, 4, "h: Ignore all header recommendations.");
+	GMT_Usage (API, 4, "f: Ignore all systematic fixes recommendations.");
+	GMT_Usage (API, 4, "n: Ignore data record bitflags pertaining to navigation (time, lon, lat).");
+	GMT_Usage (API, 4, "v: Ignore data record bitflags pertaining to data values.");
+	GMT_Usage (API, 4, "s: Ignore data record bitflags pertaining to data slopes (gradients).");
+	GMT_Usage (API, 4, "Use -DE to ignore the verification status of the e77 file [Default requires verification to be Y]. "
+		"Note: Previous E77 information will be removed prior to processing this E77 information.");
+	GMT_Usage (API, 3, "g: Sample a GMT grid along track (also see -n; use -R to select a sub-region). "
+		"Append filename of the GMT grid.");
+	GMT_Usage (API, 3, "i: Sample a Sandwell/Smith *.img Mercator grid along track (also see -n; use -R to select a sub-region). ");
+	gmt_img_syntax (API->GMT, 4);
+	GMT_Usage (API, 3, "n: Give filename with (rec_no, data) for a new column.  We expect a two-column file "
+		"with record numbers (0 means 1st row) in first column and data values in 2nd.  Only one cruise can be set. "
+		"If filename is - we read from standard input.  Only records with matching record numbers will have data assigned.");
+	GMT_Usage (API, 3, "t: Give filename with (abstime, data) for a new column.  We expect a two-column file "
+		"with dateTclock strings in first column and data values in 2nd.  Only one cruise can be set. "
+		"If filename is - we read from standard input.  Only records with matching times will have data assigned.");
+	GMT_Usage (API, 3, "T: Same as t but we interpolate between the time, data pairs to fill in all data records.");
+	GMT_Usage (API, 1, "\n-D<name1>,<name2>,...");
+	GMT_Usage (API, -2, "Delete the columns listed from all the cruise data files. "
+		"The columns are removed before any data are added.  It is not a substitute for -A...+f. "
+		"However, sometimes the shape of new data demands the old to be deleted first (you will be told).");
+	GMT_Usage (API, 1, "\n-E<no_char>");
+	GMT_Usage (API, -2, "Give character used to fill empty/missing string columns [9].");
+	GMT_Usage (API, 1, "\n-F Force mode.  This allows you to even replace the standard MGD77 columns [only extended columns can be changed].");
+	GMT_Usage (API, 1, "\n-I<abbrev>/<name>/<units>/<size>/<scale>/<offset>/\"comment\"");
+	GMT_Usage (API, -2, "In addition to the file information above, you must also specify column information:");
+	GMT_Usage (API, 3, "abbrev:  Short, abbreviated word (lower case only), like satfaa (%d char max).", MGD77_COL_ABBREV_LEN);
+	GMT_Usage (API, 3, "name:    Descriptive name, like \"Geosat/ERS-1 Free-air gravity\" (%d char max).", MGD77_COL_NAME_LEN);
+	GMT_Usage (API, 3, "units:   Units for the column (e.g., mGal, gamma, km) (%d char max).", MGD77_COL_NAME_LEN);
+	GMT_Usage (API, 3, "size:    Either t(ext), b(yte), s(hort), f(loat), i(nt), or d(ouble).");
+	GMT_Usage (API, 3, "scale:   Multiply data by this scale before writing to mgd77+ file.");
+	GMT_Usage (API, 3, "offset:  Add after scaling before writing to mgd77+ file.");
+	GMT_Usage (API, 3, "comment: Any text (in double quotes) for information about column (%d char max).", MGD77_COL_COMMENT_LEN);
+	GMT_Usage (API, -2, "Note: Option -I is ignored by -Ae. "
+		"Note for text: Interpolation is not allowed, and \"not-a-string\" is created from -E.");
+	GMT_Usage (API, 1, "\n-N%s", GMT_LEN_UNITS2_DISPLAY);
+	GMT_Usage (API, -2, "Append your choice for distance unit (if -Ad|D are set). Choose among "
+		"m(e)ter, (f)oot, (k)m, (M)ile, (n)autical mile, or s(u)rvey foot [Default is -Nk].");
+	GMT_Usage (API, -2, "Note: See -j for selecting distance calculation procedure.");
 	GMT_Option (API, "Rg,V,bi,di,j,n,.");
 
 	return (GMT_MODULE_USAGE);
@@ -335,7 +350,7 @@ static int parse (struct GMT_CTRL *GMT, struct MGD77MANAGE_CTRL *Ctrl, struct GM
 			/* Processes program-specific parameters */
 
 			case 'A':	/* Adding a new column */
-				Ctrl->A.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->A.active);
 				k = 0;
 				if (opt->arg[k] == '+') {	/* Deprecated way of doing _f */
 					Ctrl->A.replace = true;
@@ -437,28 +452,29 @@ static int parse (struct GMT_CTRL *GMT, struct MGD77MANAGE_CTRL *Ctrl, struct GM
 				}
 				break;
 			case 'D':	/* Columns to delete */
-				Ctrl->D.active = true;
-				Ctrl->D.file = strdup (opt->arg);
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->D.active);
+				n_errors += gmt_get_required_string (GMT, opt->arg, opt->option, 0, &Ctrl->D.file);
 				break;
 
 			case 'E':	/* character to generate no-string value */
-				Ctrl->E.active = true;
-				Ctrl->E.value = opt->arg[0];
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->E.active);
+				n_errors += gmt_get_required_char (GMT, opt->arg, opt->option, 0, &Ctrl->E.value);
 				break;
 
 			case 'F':	/* Force mode */
-				Ctrl->F.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->F.active);
+				n_errors += gmt_get_no_argument (GMT, opt->arg, opt->option, 0);
 				break;
 
 			case 'I':	/* Column attribute information */
-				Ctrl->I.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->I.active);
 				n_errors += mgd77manage_decode_I_options (GMT, opt->arg, Ctrl->I.c_abbrev, Ctrl->I.c_name, Ctrl->I.c_units,
 				                              &Ctrl->I.c_size, Ctrl->I.c_comment, Ctrl->A.parameters);
 				break;
 
 			case 'N':	/* Set distance units */
-				Ctrl->N.active = true;
-				Ctrl->N.code[0] = opt->arg[0];
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->N.active);
+				n_errors += gmt_get_required_char (GMT, opt->arg, opt->option, 0, &Ctrl->N.code[0]);
 				if (Ctrl->N.code[0] == 'm' && gmt_M_compat_check (GMT, 4)) {
 					GMT_Report (API, GMT_MSG_COMPAT, "Option -N: Unit m for miles is deprecated; use unit M instead\n");
 					Ctrl->N.code[0] = 'M';
@@ -470,7 +486,7 @@ static int parse (struct GMT_CTRL *GMT, struct MGD77MANAGE_CTRL *Ctrl, struct GM
 				break;
 
 			default:	/* Report bad options */
-				n_errors += gmt_default_error (GMT, opt->option);
+				n_errors += gmt_default_option_error (GMT, opt);
 				break;
 		}
 	}
@@ -666,7 +682,7 @@ EXTERN_MSC int GMT_mgd77manage (void *V_API, int mode, void *args) {
 		colvalue = gmt_M_memory (GMT, NULL, n_alloc, double);
 		if (two_cols) {	/* Got an abscissae column as well (dnt: d = dist, n = rec number, t = time) */
 			coldnt = gmt_M_memory (GMT, NULL, n_alloc, double);
-			gmt_set_column (GMT, GMT_IN, GMT_X, Ctrl->A.kind);
+			gmt_set_column_type (GMT, GMT_IN, GMT_X, Ctrl->A.kind);
 		}
 		if (strings && !two_cols) {	/* Must read strings directly from file since GMT->current.io.input would barf */
 			ok_to_read = false;
@@ -763,6 +779,9 @@ EXTERN_MSC int GMT_mgd77manage (void *V_API, int mode, void *args) {
 
 		if (MGD77_Read_File (GMT, list[argno], &In, D)) {
 			GMT_Report (API, GMT_MSG_ERROR, "Failure while reading data set for cruise %s\n", list[argno]);
+			MGD77_Free_Dataset (GMT, &D);
+			gmt_M_free (GMT, colvalue);
+			if (tmp_string) gmt_M_free (GMT, tmp_string);
 			Return (GMT_DATA_READ_ERROR);
 		}
 
@@ -773,14 +792,16 @@ EXTERN_MSC int GMT_mgd77manage (void *V_API, int mode, void *args) {
 
 		if (Ctrl->A.mode != MODE_e && column != MGD77_NOT_SET) {	/* A column with same abbreviation is already present in the file */
 			if (set == MGD77_M77_SET && !Ctrl->F.active) {
-				GMT_Report (API, GMT_MSG_ERROR, "Column %s is part of the standard MGD77 set and cannot be removed unless you use -F!\n",
-				            Ctrl->I.c_abbrev);
+				GMT_Report (API, GMT_MSG_ERROR, "Column %s is part of the standard MGD77 set and cannot be removed unless you use -F!\n", Ctrl->I.c_abbrev);
+				MGD77_Free_Dataset (GMT, &D);
+				if (tmp_string) gmt_M_free (GMT, tmp_string);
 				Return (GMT_RUNTIME_ERROR);
 			}
 			if (!Ctrl->A.replace) {
 				GMT_Report (API, GMT_MSG_ERROR, "A columned named %s is already present in %s.  use -A+ to overwrite [default is to skip]\n",
 				            Ctrl->I.c_abbrev, list[argno]);
 				MGD77_Free_Dataset (GMT, &D);	/* Free memory already allocated by MGD77_Read_File for this aborted effort */
+				if (tmp_string) gmt_M_free (GMT, tmp_string);
 				continue;
 			}
 			n_dims = (D->H.info[In.order[column].set].col[In.order[column].item].constant) ? 0 : 1;
@@ -837,6 +858,8 @@ EXTERN_MSC int GMT_mgd77manage (void *V_API, int mode, void *args) {
 
 			sprintf (oldfile, "%s.old", In.path);
 			if (gmt_rename_file (GMT, In.path, oldfile, GMT_RENAME_FILE)) {
+				MGD77_Free_Dataset (GMT, &D);
+				if (tmp_string) gmt_M_free (GMT, tmp_string);
 				Return (GMT_RUNTIME_ERROR);
 			}
 
@@ -851,6 +874,8 @@ EXTERN_MSC int GMT_mgd77manage (void *V_API, int mode, void *args) {
 
 			if (MGD77_Write_File (GMT, In.path, &In, D)) {	/* Create the new, slimmer file */
 				GMT_Report (API, GMT_MSG_ERROR, "Failure while writing slimmer version of %s\n", list[argno]);
+				MGD77_Free_Dataset (GMT, &D);
+				if (tmp_string) gmt_M_free (GMT, tmp_string);
 				Return (GMT_DATA_WRITE_ERROR);
 			}
 
@@ -858,6 +883,8 @@ EXTERN_MSC int GMT_mgd77manage (void *V_API, int mode, void *args) {
 
 			if (gmt_remove_file (GMT, oldfile))	{
 				GMT_Report (API, GMT_MSG_ERROR, "Failure while removing the old version of %s\n", list[argno]);
+				MGD77_Free_Dataset (GMT, &D);
+				if (tmp_string) gmt_M_free (GMT, tmp_string);
 				Return (GMT_RUNTIME_ERROR);
 			}
 
@@ -870,6 +897,7 @@ EXTERN_MSC int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			D = MGD77_Create_Dataset (GMT);
 			if (MGD77_Read_File (GMT, list[argno], &In, D)) {
 				GMT_Report (API, GMT_MSG_ERROR, "Failure while reading data set for cruise %s\n", list[argno]);
+				MGD77_Free_Dataset (GMT, &D);
 				Return (GMT_DATA_READ_ERROR);
 			}
 			if (reset_column)
@@ -1210,7 +1238,7 @@ EXTERN_MSC int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			flags = gmt_M_memory (GMT, NULL, D->H.n_records, unsigned int);
 			n_E77_flags = n_E77_headers = n_E77_scales = n_E77_offsets = n_E77_recalcs = 0;
 
-			MGD77_nc_status (GMT, nc_open (In.path, NC_WRITE, &In.nc_id));	/* Open the file */
+			MGD77_nc_status (GMT, gmt_nc_open (GMT, In.path, NC_WRITE, &In.nc_id));	/* Open the file */
 			MGD77_nc_status (GMT, nc_redef (In.nc_id));				/* Enter define mode */
 			(void)MGD77_Remove_E77 (GMT, &In);				/* Remove any previously revised header parameters */
 			while (gmt_fgets (GMT, line, GMT_BUFSIZ, fp_e) && strncmp (line, "# Errata: Data", 14U)) {	/* Read until we get to data record section */
@@ -1532,7 +1560,7 @@ EXTERN_MSC int GMT_mgd77manage (void *V_API, int mode, void *args) {
 
 		if (MGD77_Open_File (GMT, list[argno], &In, MGD77_UPDATE_MODE)) return (-1);	/* Only creates the full path to the new file */
 
-		MGD77_nc_status (GMT, nc_open (In.path, NC_WRITE, &In.nc_id));	/* Open the file */
+		MGD77_nc_status (GMT, gmt_nc_open (GMT, In.path, NC_WRITE, &In.nc_id));	/* Open the file */
 		MGD77_nc_status (GMT, nc_redef (In.nc_id));				/* Enter define mode */
 
 		dims[0] = In.nc_recid;	dims[1] = LEN;
